--[[
Created by Grid2 original authors, modified by Michael
--]]

local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("Grid2")

local function Bar_CreateHH(self, parent)
	local media = LibStub("LibSharedMedia-3.0", true)
	local texture = media and media:Fetch("statusbar", self.dbx.texture) or "Interface\\Addons\\Grid2\\gradient32x32"

	local bar = parent[self.name] or CreateFrame("StatusBar", nil, parent)
	bar:SetOrientation(self.orientation or Grid2Frame.db.profile.orientation)
	bar:SetStatusBarColor(0,0,0,0)
	bar:SetMinMaxValues(0, 1)
	bar:SetValue(0)
	bar:SetStatusBarTexture(texture)
	parent[self.name] = bar
	if not bar:IsShown() then bar:Show() end
end

local function Bar_Layout(self, parent)
	local Bar, BarBG = parent[self.name], parent.BarBG
	Bar:ClearAllPoints()
	Bar:SetFrameLevel(parent:GetFrameLevel() + self.frameLevel)
	Bar:SetPoint(self.anchor, parent.BarBG, self.anchorRel, self.offsetx, self.offsety)
    Bar:SetWidth(self.width or BarBG:GetWidth())
	Bar:SetHeight(self.height or BarBG:GetHeight())
end

local function Bar_GetBlinkFrame(self, parent)
	return parent[self.name]
end

local function Bar_OnUpdate(self, parent, unit, status)
	parent[self.name]:SetValue(status and status:GetPercent(unit) or 0)
end

local function Bar_SetOrientation(self, parent, orientation)
	parent[self.name]:SetOrientation(orientation or Grid2Frame.db.profile.orientation)
end

local function Bar_SetTexture(self, parent, texture)
	parent[self.name]:SetStatusBarTexture(texture)
end

local function Bar_Disable(self, parent)
	parent[self.name]:Hide()
	self.Layout = nil
	self.OnUpdate = nil
end

local function Bar_UpdateDB(self, dbx)
	local l= dbx.location
	self.frameLevel = dbx.level or 1
	self.anchor = l.point
	self.anchorRel = l.relPoint
	self.offsetx = l.x
	self.offsety = l.y
	self.width= dbx.width
	self.height= dbx.height
	self.orientation= dbx.orientation
	self.Create = Bar_CreateHH
	self.Layout = Bar_Layout
	self.GetBlinkFrame = Bar_GetBlinkFrame
	self.OnUpdate = Bar_OnUpdate
	self.SetOrientation = Bar_SetOrientation
	self.SetTexture = Bar_SetTexture
	self.Disable = Bar_Disable	
	self.UpdateDB = Bar_UpdateDB
	self.dbx = dbx
end

local function BarColor_Create(self, parent)
end

local function BarColor_Layout(self, parent)
end

local function BarColor_OnUpdate(self, parent, unit, status)
	if status then
		self:SetBarColor(parent, status:GetColor(unit))
	else
		self:SetBarColor(parent, 0, 0, 0, 0)
	end
end

local function BarColor_SetBarColor(self, parent, r, g, b, a)
	local Bar= parent[self.BarName]
	Bar:SetStatusBarColor(r, g, b, a)
end

local function BarColor_SetBarColorInverted(self, parent, r, g, b, a)
	local Bar, BarBG = parent[self.BarName], parent.BarBG
	Bar:SetStatusBarColor(0, 0, 0, 0.8)
	BarBG:SetVertexColor(r, g, b, a)
end

local function BarColor_UpdateDB(self)
	if self.dbx.invertColor then
		self.SetBarColor = BarColor_SetBarColorInverted
	else
		self.SetBarColor = BarColor_SetBarColor
	end
end

local function Create(indicatorKey, dbx)
	local existingIndicator= Grid2.indicators[indicatorKey]
	local Bar= existingIndicator or Grid2.indicatorPrototype:new(indicatorKey)
	Bar_UpdateDB(Bar,dbx)
	Grid2:RegisterIndicator(Bar, { "percent" })

	local colorKey = indicatorKey .. "-color"
	existingIndicator = Grid2.indicators[colorKey]
	local BarColor = existingIndicator or Grid2.indicatorPrototype:new(colorKey)
	BarColor.dbx = dbx
	BarColor.BarName= indicatorKey
	BarColor.Create = BarColor_Create
	BarColor.Layout = BarColor_Layout
	BarColor.OnUpdate = BarColor_OnUpdate
	BarColor.UpdateDB = BarColor_UpdateDB
	BarColor_UpdateDB(BarColor)
	Grid2:RegisterIndicator(BarColor, { "color" })

	Bar.sideKick = BarColor

	return Bar, BarColor
end

Grid2.setupFunc["bar"] = Create

--ToDo: Is there a better way to handle this dual indicator creation?
local function CreateColor(indicatorKey, dbx)
end
Grid2.setupFunc["bar-color"] = CreateColor

